
import java.io.*;
import java.net.*;
import java.util.*;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.*;

import org.apache.soap.util.xml.*;
import org.apache.soap.*;
import org.apache.soap.encoding.*;
import org.apache.soap.encoding.soapenc.*;
import org.apache.soap.rpc.*;


public class Client {

  private URL url;
  private String uri;
  private authInfo authInfo;

  public Client (String url, String uri) throws Exception {
    try {
      this.uri = uri;
      this.url = new URL(url);
    } catch (Exception e) {
      throw new Exception(e.getMessage());
    }
  }

  public Header makeAuthHeader (authInfo auth) throws Exception {
    if (auth == null) { throw new Exception("Oops, you are not logged in. Please login first"); }
    DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
    dbf.setNamespaceAware(true);
    dbf.setValidating(false);
    DocumentBuilder db = dbf.newDocumentBuilder();
    Document doc = db.newDocument();
    Element authEl = doc.createElementNS("http://www.soaplite.com/authInfo", "auth:authInfo");
    Element emailEl = doc.createElement("email");
    emailEl.appendChild(doc.createTextNode(auth.getEmail()));
    Element signatureEl = doc.createElement("signature");
    signatureEl.setAttribute("xmlns:enc", Constants.NS_URI_SOAP_ENC);
    signatureEl.setAttribute("xsi:type", "enc:base64");
    signatureEl.appendChild(doc.createTextNode(Base64.encode(auth.getSignature())));
    Element memberIdEl = doc.createElement("memberID");
    memberIdEl.appendChild(doc.createTextNode(String.valueOf(auth.getMemberID())));
    Element timeEl = doc.createElement("time");
    timeEl.appendChild(doc.createTextNode(String.valueOf(auth.getTime())));
    authEl.appendChild(emailEl);
    authEl.appendChild(signatureEl);
    authEl.appendChild(memberIdEl);
    authEl.appendChild(timeEl);
    Vector headerEntries = new Vector();
    headerEntries.add(authEl);
    Header header = new Header();
    header.setHeaderEntries(headerEntries);
    return header;
  }

  private Call initCall () {
    Call call = new Call();
    call.setEncodingStyleURI(Constants.NS_URI_SOAP_ENC);
    call.setTargetObjectURI(uri);
    return call;
  }

  private Object invokeCall (Call call) throws Exception {
    try {
      Response response = call.invoke(url, "");

      if (!response.generatedFault()) { 
        return response.getReturnValue() == null 
          ? null : response.getReturnValue().getValue();
      } else { 
        Fault f = response.getFault();
        throw new Exception("Fault = " + f.getFaultCode() + ", " + f.getFaultString()); 
      } 
    } catch (SOAPException e) { 
      throw new Exception("SOAPException = " + e.getFaultCode() + ", " + e.getMessage()); 
    }
  }

  public void login (String email, String password) throws Exception {
    Call call = initCall();

    SOAPMappingRegistry smr = new SOAPMappingRegistry();
    BeanSerializer beanSer = new BeanSerializer();
    smr.mapTypes(Constants.NS_URI_SOAP_ENC,
                 new QName("http://www.soaplite.com/Publisher", "authInfo"),
                 authInfo.class, beanSer, beanSer);

    Vector params = new Vector ();
    params.add(new Parameter("email", String.class, email, null));
    params.add(new Parameter("password", String.class, password, null));
    call.setParams(params);
    call.setMethodName("login");
    call.setSOAPMappingRegistry(smr);

    authInfo = (authInfo) invokeCall(call);

    System.out.println(authInfo.getEmail() + " logged in.");
  }

  public void register (String email, String password, String firstName, String lastName, String title, String company, String url) throws Exception {
    Call call = initCall();

    Vector params = new Vector ();
    params.add(new Parameter("email", String.class, email, null));
    params.add(new Parameter("password", String.class, password, null));
    params.add(new Parameter("firstName", String.class, firstName, null));
    params.add(new Parameter("lastName", String.class, lastName, null));
    if (url != null)     params.add(new Parameter("url", String.class, url, null));
    if (title != null)   params.add(new Parameter("title", String.class, title, null));
    if (company != null) params.add(new Parameter("company", String.class, company, null));
    call.setParams(params);
    call.setMethodName("register");
    invokeCall(call);
    System.out.println("Registered.");
  }

  public void postItem (String type, String title, String description) throws Exception {
    Call call = initCall();
    Vector params = new Vector ();
    params.add(new Parameter("type", String.class, type, null));
    params.add(new Parameter("title", String.class, title, null));
    params.add(new Parameter("description", String.class, description, null));
    call.setParams(params);
    call.setMethodName("postItem");
    call.setHeader(makeAuthHeader(authInfo));
    Integer itemID = (Integer)invokeCall(call);
    System.out.println("Posted item " + itemID + ".");
  }

  public void removeItem (Integer itemID) throws Exception {
    Call call = initCall();
    Vector params = new Vector ();
    params.add(new Parameter("itemID", Integer.class, itemID, null));
    call.setParams(params);
    call.setMethodName("removeItem");
    call.setHeader(makeAuthHeader(authInfo));
    invokeCall(call);
    System.out.println("Removed item " + itemID + ".");
  }

  public void browse (String type, String format, Integer maxRows) throws Exception {
    Call call = initCall();
    Vector params = new Vector ();
    params.add(new Parameter("format", String.class, format != null ? format : "XMLstr", null));
    if (type != null)    params.add(new Parameter("type", String.class, type, null));
    if (maxRows != null) params.add(new Parameter("maxRows", Integer.class, maxRows, null));
    call.setParams(params);
    call.setMethodName("browse");
    System.out.println((String)invokeCall(call));
  }

  public static void main(String[] args) {

    String myname = Client.class.getName();

    if (args.length < 1) {
      System.err.println("Usage:\n  java " + myname +
        " SOAP-router-URL");
      System.exit (1);
    }

    try {
      Client client = new Client(args[0], "http://www.soaplite.com/Publisher");

      InputStream in = System.in;
      InputStreamReader isr = new InputStreamReader(in);
      BufferedReader br = new BufferedReader(isr);
      String action = null;
      while (!("quit".equals(action))) {
         System.out.print("> ");
         action = br.readLine();


         if ("register".equals(action)) {

             String email = null;
             String password = null;
             String firstName = null;
             String lastName = null;
             String title = null;
             String company = null;
             String url = null;

             System.out.print("\n\nIn order to register, you must answer the following questions.");
             System.out.print("\n\nWhat is your email address: ");
             email = br.readLine();
             System.out.print("\nWhat is your first name: ");
             firstName = br.readLine();
             System.out.print("\nWhat is your last name: ");
             lastName = br.readLine();
             System.out.print("\nWhat is your job title: ");
             title = br.readLine();
             System.out.print("\nWhat company do you work for: ");
             company = br.readLine();
             System.out.print("\nWhat is your company or personal URL: ");
             url = br.readLine();
             System.out.print("\nFinally, what password do you want to use: ");
             password = br.readLine();

             System.out.println("\nAttempting to register....");
             client.register(email, password, firstName, lastName, title, company, url);
             System.out.println();
         }

         if ("login".equals(action)) {
             String id = null;
             String pwd = null;
             
             System.out.print("\n\nWhat is your user id: ");
             id = br.readLine();
             System.out.print("\nWhat is your password: ");
             pwd = br.readLine();

             System.out.println("\nAttempting to login....");
             client.login(id,pwd);
             System.out.println();
         }

         if ("post".equals(action)) {

             String type = null;
             String title = null;
             String desc = null;

             System.out.print("\n\nWhat type of item [1 = News, 2 = Article, 3 = Resource]: ");
             type = br.readLine();
             if (type.equals("1")) type = "news";
             if (type.equals("2")) type = "article";
             if (type.equals("3")) type = "resource";
             System.out.println("\nWhat is the title: ");
             title = br.readLine();
             System.out.println("\nWhat is the description: ");
             desc = br.readLine();

             System.out.println("\nAttempting to post item....");
             client.postItem(type, title, desc);
             System.out.println();
         }

         if ("remove".equals(action)) {
             System.out.print("\n\nPlease enter the numeric ID of the item to remove: ");
             String id = br.readLine();
             try {
                System.out.println("\nAttempting to remove item....");
                client.removeItem(Integer.valueOf(id));
             } catch (Exception ex) {
                System.out.println("\nCould not remove item!");
             }
             System.out.println();
         }

         if ("browse".equals(action)) {
             System.out.print("\n\nWhat is the maximum number of rows to return (blank to return all): ");
             String mRows = br.readLine();
             System.out.print("\nType of resource to browse ([0] = All, [1] = News, [2] = Article, [3] = Resource): ");
             String type = br.readLine();
             if (type.equals("0")) type = "all";
             if (type.equals("1")) type = "news";
             if (type.equals("2")) type = "article";
             if (type.equals("3")) type = "resource";
             System.out.print("\nHow would you like to see the results ([1] = XML, [2] = RSS): ");
             String format = br.readLine();
             if (format.equals("1")) format = "XMLstr";
             if (format.equals("2")) format = "RSSstr";

             System.out.println("\nAttempting to browse....");
             try {
                 Integer ival = null;
                 if (!("".equals(mRows))) {
                     ival = Integer.valueOf(mRows);
                 }
                 client.browse(type, format, ival);
             } catch (Exception ex) { 
                System.out.println(ex);
                System.out.println("\nCould not browse!"); 
             }
         }
         
         if ("help".equals(action)) {
             System.out.println("\nActions: register | login | post | remove | browse");
         }
      }
    } catch (Exception e) {
      System.err.println("Caught Exception: " + e.getMessage());
    }
  }
}
