# ======================================================================
# Copyright (C) 2001 Paul Kulchenko (paulclinger@yahoo.com)
# ======================================================================

package Publisher;

use strict;

# ======================================================================

package Publisher::DB;

use DBI;

my $dbh;

sub dbh {
  shift;
  unless ($dbh) {
    # check parameter for proper format? 
    # security issue? probably yes, for DBD::File, DBD::CSV, etc
    $dbh = DBI->connect(shift || "DBI:CSV:f_dir=.;csv_sep_char=\0");
    # $dbh = DBI->connect(shift || "DBI:CSV:f_dir=.;csv_sep_char=\0");
    $dbh->{'RaiseError'} = 1;
  }
  return $dbh;
}

END { $dbh->disconnect if $dbh; }

sub create {
  my $dbh = shift->dbh;

  $dbh->do($_) foreach split /;/, '

  CREATE TABLE members (
    memberID   integer,
    email      char(100),
    password   char(25),
    firstName  char(50),
    lastName   char(50),
    title      char(50),
    company    char(50),
    url        char(255),
    subscribed integer
  );

  CREATE TABLE items (
    itemID	integer,
    memberID	integer,
    type        integer,
    title  	char(255),
    description char(512),
    postStamp	integer
  )

';

}

sub insert_member {
  my $dbh = shift->dbh;
  my $newMemberID = 1 + $dbh->selectrow_array("SELECT memberID FROM members ORDER BY memberID DESC");

  my %parameters = (@_, memberID => $newMemberID, subscribed => 0);
  my $names = join ', ', keys %parameters;
  my $placeholders = join ', ', ('?') x keys %parameters;

  $dbh->do("INSERT INTO members ($names) VALUES ($placeholders)", {}, 
           values %parameters);
  return $newMemberID;
}

sub select_member {
  my $dbh = shift->dbh;
  my %parameters = @_;

  my $where = join ' AND ', map {"$_ = ?"} keys %parameters;
  $where = "WHERE $where" if $where;

  # returns row in array context and first element (memberID) in scalar
  return $dbh->selectrow_array("SELECT * FROM members $where", {}, values %parameters);
}

sub update_member {
  my $dbh = shift->dbh;
  my($memberID, %parameters) = @_;

  my $set = join ', ', map {"$_ = ?"} keys %parameters;

  $dbh->do("UPDATE members SET $set WHERE memberID = ?", {}, 
           values %parameters, $memberID);
  return $memberID;
}

sub insert_item {
  my $dbh = shift->dbh;
  my $newItemID = 1 + $dbh->selectrow_array("SELECT itemID FROM items ORDER BY itemID DESC");

  my %parameters = (@_, itemID => $newItemID, postStamp => time());
  my $names = join ', ', keys %parameters;
  my $placeholders = join ', ', ('?') x keys %parameters;

  $dbh->do("INSERT INTO items ($names) VALUES ($placeholders)", {},
           values %parameters);

  return $newItemID;
}

sub select_item {
  my $dbh = shift->dbh;
  my %parameters = @_;

  my $where = join ' AND ', map {"$_ = ?"} keys %parameters;

  return $dbh->selectrow_array("SELECT * FROM items WHERE $where", {}, values %parameters);
}

sub select_all_items {
  my $dbh = shift->dbh;
  my %parameters = @_;

  my $where = join ' AND ', map {"$_ = ?"} keys %parameters;
  $where = "WHERE $where" if $where;

  return $dbh->selectall_arrayref("SELECT type, title, description, postStamp, memberID FROM items $where", {}, values %parameters);
}

sub delete_item {
  my $dbh = shift->dbh;
  my $itemID = shift;

  $dbh->do('DELETE FROM items WHERE itemID = ?', {}, $itemID);
  return $itemID;
}

# ======================================================================

package Publisher;

use POSIX qw(strftime);

@Publisher::ISA = qw(SOAP::Server::Parameters);

# ----------------------------------------------------------------------
# private functions
# ----------------------------------------------------------------------

use Digest::MD5 qw(md5); 

my $calculateAuthInfo = sub {
  return md5(join '', 'unique (yet persistent) string', @_);
};

my $checkAuthInfo = sub {
  my $authInfo = shift;
  my $signature = $calculateAuthInfo->(@{$authInfo}{qw(memberID email time)});
  die "Authentication information is not valid\n" if $signature ne $authInfo->{signature};
  die "Authentication information is expired\n" if time() > $authInfo->{time};
  return $authInfo->{memberID};
};

my $makeAuthInfo = sub {
  my($memberID, $email) = @_;
  my $time = time()+20*60;
  my $signature = $calculateAuthInfo->($memberID, $email, $time);
  return +{memberID => $memberID, time => $time, email => $email, signature => $signature};
};

# ----------------------------------------------------------------------
# public functions
# ----------------------------------------------------------------------

sub register {
  my $self = shift;
  my $envelope = pop;
  my %parameters = %{$envelope->method() || {}};

  die "Wrong parameters: register(email, password, firstName, lastName [, title][, company][, url])\n"
    unless 4 == map {defined} @parameters{qw(email password firstName lastName)};

  my $email = $parameters{email};
  die "Member with email ($email) already registered\n" 
    if Publisher::DB->select_member(email => $email);
  return Publisher::DB->insert_member(%parameters);
}

sub modify {
  my $self = shift;
  my $envelope = pop;
  my %parameters = %{$envelope->method() || {}};

  my $memberID = $checkAuthInfo->($envelope->valueof('//authInfo'));
  Publisher::DB->update_member($memberID, %parameters);
  return;
}

sub login { 
  my $self = shift;
  my %parameters = %{pop->method() || {}};

  my $email = $parameters{email};
  my $memberID = Publisher::DB->select_member(email => $email, password => $parameters{password});
  die "Credentials are wrong\n" unless $memberID;
  return bless $makeAuthInfo->($memberID, $email) => 'authInfo';
}

sub subscribe { 
  my $self = shift;
  my $memberID = $checkAuthInfo->(pop->valueof('//authInfo'));

  Publisher::DB->update_member($memberID, subscribed => 1);
  return;
}

sub unsubscribe { 
  my $self = shift;
  my $memberID = $checkAuthInfo->(pop->valueof('//authInfo'));

  Publisher::DB->update_member($memberID, subscribed => 0);
  return;
}

my %type2code = (news => 1, article => 2, resource => 3);
my %code2type = reverse %type2code;

sub postItem {
  my $self = shift;
  my $envelope = pop;
  my $memberID = $checkAuthInfo->($envelope->valueof('//authInfo'));
  my %parameters = %{$envelope->method() || {}};

  die "Wrong parameter(s): postItem(type, title, description)\n" 
    unless 3 == map {defined} @parameters{qw(type title description)};

  $parameters{type} = $type2code{lc $parameters{type}} or die "Wrong type of item ($parameters{type})\n";
  return Publisher::DB->insert_item(memberID => $memberID, %parameters);
}

sub removeItem {
  my $self = shift;
  my $memberID = $checkAuthInfo->(pop->valueof('//authInfo'));
  die "Wrong parameter(s): removeItem(itemID)\n" unless @_ == 1;

  my $itemID = shift;
  die "Specified item ($itemID) can't be found or removed\n" 
    unless Publisher::DB->select_item(memberID => $memberID, itemID => $itemID);
  Publisher::DB->delete_item($itemID);
  return;
}

my $browse = sub {
  my $envelope = pop;
  my %parameters = %{$envelope->method() || {}};

  my($type, $format, $maxRows, $query) = @parameters{qw(type format maxRows query)};
  $type = {all => 'all', %type2code}->{lc($type) || 'all'} or
    die "Wrong type of item ($type)\n";

  $maxRows ||= 25;
  $format ||= 'XML';
  my $items = Publisher::DB->select_all_items($type ne 'all' ? (type => $type) : ());
  my %members;
  my @items = map {
    my($type, $title, $description, $date, $memberID) = @$_;
    my($email, $firstName, $lastName) = @{
      $members{$memberID} ||= [Publisher::DB->select_member(memberID => $memberID)]
    }[1,3,4];
    +{
      $format =~ /^XML/ ? (
        type        => $code2type{$type}, 
        title       => $title, 
        description => $description,
        date        => strftime("%Y-%m-%d", gmtime($date)),
        creator     => "$firstName $lastName ($email)"
      ) : (
        category    => $code2type{$type}, 
        title       => "$title by $firstName $lastName ($email) on " . strftime("%Y-%m-%d", gmtime($date)), 
        description => $description,
      )
    }
  } @{$items}[0..(!$query && $maxRows <= $#$items ? $maxRows-1 : $#$items)];
  if ($query) {
    my $regexp = join '', map {
      /\s+and\s+/io ? '&&' : /\s+or\s+/io ? '||' : /[()]/ ? $_ : $_ ? '/' . quotemeta($_) . '/o' : ''
    } split /(\(|\)|\s+and\s+|\s+or\s+)/io, $query;
    eval "*checkfor = sub { for (\@_) { return 1 if $regexp; } return }" or die;
    @items = grep {checkfor(values %$_)} @items;
    splice(@items, $maxRows <= $#items ? $maxRows : $#items+1);
  }
  return $format =~ /^(XML|RSS)str$/ 
    ? SOAP::Serializer
        -> autotype(0)
        -> readable(1)
        -> serialize(SOAP::Data->name(($1 eq 'XML' ? 'itemList' : 'channel') 
                     => \SOAP::Data->name(item => @items)))
    : [@items];
};

sub browse {
  my $self = shift;
  return SOAP::Data->name(browse => $browse->(@_));
}

sub search {
  my $self = shift;
  return SOAP::Data->name(search => $browse->(@_));
}

# ======================================================================

1;
